@@ -19,6 +19,7 @@ from .types import (
     TFNumeric,
     TorchNumeric,
     JAXNumeric,
+    RandomState,
 )
 from .util import abstract
 
@@ -90,6 +91,8 @@ __all__ = [
     "le",
     "gt",
     "ge",
+    "eq",
+    "ne",
     "bvn_cdf",
     "cond",
     "where",
@@ -183,7 +186,7 @@ def _jit_run(
     f: FunctionType,
     compilation_cache: dict,
     jit_kw_args: dict,
-    *args: Numeric,
+    *args: Union[Numeric, RandomState],
     **kw_args
 ):  # pragma: no cover
     pass
@@ -403,8 +406,8 @@ def zero(dtype: DType):
 
 
 @dispatch
-def zero(ref: Numeric):
-    return zero(B.dtype(ref))
+def zero(*refs):
+    return zero(B.dtype(*refs))
 
 
 @dispatch
@@ -421,8 +424,8 @@ def one(dtype: DType):
 
 
 @dispatch
-def one(ref: Numeric):
-    return one(B.dtype(ref))
+def one(*refs):
+    return one(B.dtype(*refs))
 
 
 @dispatch
@@ -1224,6 +1227,34 @@ def ge(a, b):  # pragma: no cover
     """
 
 
+@dispatch
+@abstract(promote=2)
+def ne(a, b):  # pragma: no cover
+    """Check whether one tensor is not equal to another.
+
+    Args:
+        a (tensor): First tensor.
+        b (tensor): Second tensor.
+
+    Returns:
+        tensor[bool]: `a` is not equal to `b`.
+    """
+
+
+@dispatch
+@abstract(promote=2)
+def eq(a, b):  # pragma: no cover
+    """Check whether one tensor is equal to another.
+
+    Args:
+        a (tensor): First tensor.
+        b (tensor): Second tensor.
+
+    Returns:
+        tensor[bool]: `a` is equal to `b`.
+    """
+
+
 @dispatch
 @abstract(promote=3)
 def bvn_cdf(a, b, c):
