@@ -8,7 +8,7 @@ from plum import Union
 from . import B, dispatch, Numeric
 from .custom import jax_register
 from ..custom import bvn_cdf, i_bvn_cdf, s_bvn_cdf, i_s_bvn_cdf
-from ..types import JAXDType, JAXNumeric, NPNumeric, Number, Int, _jax_tracer
+from ..types import JAXDType, JAXNumeric, NPNumeric, Number, Int, _jax_tracer, JAXRandomState
 
 __all__ = []
 
@@ -23,7 +23,7 @@ def _jit_run(
     f: FunctionType,
     compilation_cache: dict,
     jit_kw_args: dict,
-    *args: Numeric,
+    *args: Union[Numeric, JAXRandomState],
     **kw_args,
 ):
     if "jax" not in compilation_cache:
@@ -292,6 +292,16 @@ def ge(a: Numeric, b: Numeric):
     return jnp.greater_equal(a, b)
 
 
+@dispatch
+def eq(a: Numeric, b: Numeric):
+    return jnp.equal(a, b)
+
+
+@dispatch
+def ne(a: Numeric, b: Numeric):
+    return jnp.not_equal(a, b)
+
+
 _bvn_cdf = jax_register(bvn_cdf, i_bvn_cdf, s_bvn_cdf, i_s_bvn_cdf)
 
 
