@@ -4,16 +4,14 @@ from .. import dispatch as dispatch_original
 from ..shape import dispatch_unwrap_dimensions
 from ..types import Number, NPNumeric
 
-dispatch_unwrapped = dispatch_unwrap_dimensions(dispatch_original)
-
-
 # All methods here should have precedence, because NumPy forms the base of everything.
-def dispatch(*args, **kw_args):
-    kw_args["precedence"] = 1
-    return dispatch_unwrapped(*args, **kw_args)
+dispatch_original = dispatch_original(precedence=1)
 
+dispatch = dispatch_unwrap_dimensions(dispatch_original)
 
-Numeric = {Number, NPNumeric}
+from plum import Union
+
+Numeric = Union[Number, NPNumeric]
 
 from .generic import *
 from .shaping import *
