@@ -8,7 +8,7 @@ from plum import Union
 from . import dispatch, B, Numeric, TFNumeric
 from .custom import tensorflow_register
 from ..custom import bvn_cdf, s_bvn_cdf
-from ..types import TFDType, Int
+from ..types import TFDType, Int, TFRandomState
 
 __all__ = []
 
@@ -23,7 +23,7 @@ def _jit_run(
     f: FunctionType,
     compilation_cache: dict,
     jit_kw_args: dict,
-    *args: Numeric,
+    *args: Union[Numeric, TFRandomState],
     **kw_args,
 ):
     if "tensorflow" not in compilation_cache:
@@ -54,7 +54,6 @@ def imag(a: Numeric):
     return tf.math.imag(a)
 
 
-
 @dispatch
 def device(a: TFNumeric):
     return a.device
@@ -293,6 +292,16 @@ def ge(a: Numeric, b: Numeric):
     return tf.greater_equal(a, b)
 
 
+@dispatch
+def eq(a: Numeric, b: Numeric):
+    return tf.equal(a, b)
+
+
+@dispatch
+def ne(a: Numeric, b: Numeric):
+    return tf.not_equal(a, b)
+
+
 _bvn_cdf = tensorflow_register(bvn_cdf, s_bvn_cdf)
 
 
