@@ -18,6 +18,11 @@ def global_random_state(_: TorchDType):
     return torch.random.default_generator
 
 
+@dispatch
+def set_global_random_state(state: TorchRandomState):
+    torch.random.default_generator.set_state(state.get_state())
+
+
 @dispatch
 def rand(state: TorchRandomState, dtype: TorchDType, *shape: Int):
     return state, torch.rand(
